@@ -11,7 +11,7 @@ import sys
 import traceback
 from contextlib import closing
 
-__version__ = '1.6.0'
+__version__ = '1.7.0'
 
 try:
     import signalfd
@@ -25,6 +25,11 @@ try:
     InterruptedError = InterruptedError
 except NameError:  # python <= 3.2
     InterruptedError = OSError
+try:
+    BrokenPipeError = BrokenPipeError
+except NameError:  # old python
+    class BrokenPipeError(Exception):
+        pass
 if hasattr(sys, 'setswitchinterval'):
     setinterval = sys.setswitchinterval
     getinterval = sys.getswitchinterval
@@ -45,10 +50,6 @@ except ImportError:
             return getattr(__import__(mod), name)
 
 _ORIGINAL_SOCKET = _get_original('socket', 'socket')
-_ORIGINAL_FROMFD = _get_original('socket', 'fromfd')
-_ORIGINAL_FDOPEN = _get_original('os', 'fdopen')
-_ORIGINAL_DUP = _get_original('os', 'dup')
-_ORIGINAL_DUP2 = _get_original('os', 'dup2')
 try:
     _ORIGINAL_ALLOCATE_LOCK = _get_original('thread', 'allocate_lock')
 except ImportError:  # python 3
@@ -267,7 +268,7 @@ def handle_connection_exec(client):
         raise ExitExecLoop()
 
     client.settimeout(None)
-    fh = os.fdopen(client.detach() if hasattr(client, 'detach') else client.fileno())
+    fh = client.makefile()
 
     with closing(client):
         with closing(fh):
@@ -296,13 +297,14 @@ def handle_connection_repl(client):
     if _MANHOLE.redirect_stderr:
         patches.append(('w', ('stderr', '__stderr__')))
     try:
-        client_fd = client.fileno()
         for mode, names in patches:
             for name in names:
                 backup.append((name, getattr(sys, name)))
-                setattr(sys, name, _ORIGINAL_FDOPEN(client_fd, mode, 1 if PY3 else 0))
+                setattr(sys, name, client.makefile(mode, 1 if PY3 else 0))
         try:
             handle_repl(_MANHOLE.locals)
+        except BrokenPipeError:
+            _LOG("REPL client disconnected")
         except Exception as exc:
             _LOG("REPL failed with %r." % exc)
         _LOG("DONE.")
@@ -367,7 +369,14 @@ def handle_repl(locals):
     }
     if locals:
         namespace.update(locals)
-    ManholeConsole(namespace).interact()
+    try:
+        ManholeConsole(namespace).interact()
+    finally:
+        for attribute in ['last_type', 'last_value', 'last_traceback']:
+            try:
+                delattr(sys, attribute)
+            except AttributeError:
+                pass
 
 
 class Logger(object):
